
/* spuren programm, Copyright (c) 1991 - 99
 * Thomas Feuerstein <bik@ast5.uibk.ac.at>, Idee
 * Armin Hanika, erste DOS Version 1991 
 * Peter Chiocchetti <girbal@super.tacheles.de>, dieses 98-08-08/99-01-12
 * in Anlehnung an:
 */
/* xscreensaver, Copyright (c) 1998 Jamie Zawinski <jwz@netscape.com>
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation. No representations are made about the suitability of this
 * software for any purpose. It is provided "as is" without express or 
 * implied warranty.
 *
 */

#include "screenhack.h"

#define maxRows 128
#define maxCols 100

static void move (Display *dpy, Window window, GC gc,
		  unsigned long fgc, unsigned long bgc,
		  int numRows, int numCols, int maxPix,
		  int cellWidth, int cellHeight,
		  int d[200], int i[200], int r[200])
{
  int j, k, x, y, v;

  for (j = 0; j < numRows; ++j) {
    if (i[j] + d[j] < cellWidth)
      i[j] += d[j];
    else {
      i[j] = 0;
      d[j] = (random () % maxPix) + 1;
    }
    y = j * cellHeight;

    for (k = 0; k <= numCols; ++k) {
      if ((k % 2) == r[j])
	XSetForeground(dpy, gc, fgc);
      else
	XSetForeground(dpy, gc, bgc);
      switch(k) {
      case 0:
	x = 0;
	v = i[j];
	break;
      case 1:
	x += i[j];
	v = cellWidth;
	break;
      default:
	x += cellWidth;
	if (k == numCols)
	  v = cellWidth -i[j];
	else
	  v = cellWidth;
      }
      /*       if (j + 1 == numRows) */
      /* 	XFillRectangle(dpy, window, gc, x, y, v, cellHeight - 1); */
      /*       else */
      XFillRectangle(dpy, window, gc, x, y, v, cellHeight);
    }
    if (i[j] + d[j] >= cellWidth) {
      r[j] = !r[j];
    }
  }
}


char *progclass = "Spuren";

char *defaults [] = {
  "Spuren.background:	black",		/* to placate SGI */
  "Spuren.foreground:	white",
  "*numRows:		0",
  "*numCols:		0",
  "*maxPix:	        0",
  "*hold:		2000",
  "*barcode             false",
  "*malewitsch          false",
  0
};

XrmOptionDescRec options [] = {
  { "-numRows",		".numRows",	XrmoptionSepArg, 0 },
  { "-numCols",		".numCols",	XrmoptionSepArg, 0 },
  { "-maxPix",		".maxPix",	XrmoptionSepArg, 0 },
  { "-hold",		".hold",	XrmoptionSepArg, 0 },
  { "-barcode",         ".barcode",	XrmoptionNoArg, "True" },
  { "-malewitsch",	".malewitsch",  XrmoptionNoArg, "True" },
  { 0, 0, 0, 0 }
};

void
screenhack (Display *dpy, Window window)
{
  GC gc;
  XWindowAttributes xgwa;
  unsigned long fgc, bgc;
  int hold;
  int numRows, numCols, maxPix;
  int cellWidth, cellHeight, j, k;
  int d[200], i[200], r[200];

  XGetWindowAttributes (dpy, window, &xgwa);

  fgc  = get_pixel_resource ("foreground","Foreground", dpy, xgwa.colormap);
  bgc  = get_pixel_resource ("background","Background", dpy, xgwa.colormap);
  gc  = XCreateGC(dpy, window, 0, 0);

  hold = get_integer_resource ("hold", "Integer");
  if (hold < 0) hold = 0;

  numRows = get_integer_resource ("numRows", "Integer");
  if (numRows < 2) {
    numRows = 1;
    k = random () % 7 + 1;
    for (j = 0; j < k; ++j)
      numRows *= 2;
  }
  else if (numRows > maxRows) numRows = maxRows;

  numCols = get_integer_resource ("numCols", "Integer");
  if (numCols < 2) numCols = random () % 7 + 1;
  else if (numCols > maxCols) numCols = maxCols;

  if (get_boolean_resource ("barcode", "Boolean")) {
    numRows = 128;
    numCols = 1;
  }
  else if (get_boolean_resource ("malewitsch", "Boolean")) {
    numRows = 2;
    numCols = 2;
  }

  cellWidth = xgwa.width / numCols;
  cellHeight = xgwa.height / numRows;
  numCols = xgwa.width / cellWidth;
  numRows = xgwa.height / cellHeight;

  maxPix = get_integer_resource ("maxPix", "Integer");
  if (maxPix < 1) maxPix = random () % (cellWidth / 15 < 12
					? cellWidth / 15 : 12) + 2;
  else if (maxPix > 15) maxPix = 15;

  for (j = 0; j < numRows; ++j) {
    i[j] = 0;
    r[j] = j % 2;
    d[j] = (random () % maxPix) + 1;
  }

  while (1) {
    move (dpy, window, gc, fgc, bgc, numRows, numCols, maxPix,
	  cellWidth, cellHeight, d, i, r);
    XFlush (dpy);
    usleep (hold);
  }
}
